Package: mlr3tuningspaces
Title: Search Spaces for 'mlr3'
Version: 0.6.0
Authors@R: c(
    person("Marc", "Becker", , "marcbecker@posteo.de", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0002-8115-0400")),
    person("Michel", "Lang", , "michellang@gmail.com", role = "ctb",
           comment = c(ORCID = "0000-0001-9754-0393"))
  )
Description: Collection of search spaces for hyperparameter optimization in the
  'mlr3' ecosystem. It features ready-to-use search spaces for many popular
  machine learning algorithms. The search spaces are from scientific articles
  and work for a wide range of data sets.
License: LGPL-3
URL: https://mlr3tuningspaces.mlr-org.com,
        https://github.com/mlr-org/mlr3tuningspaces
BugReports: https://github.com/mlr-org/mlr3tuningspaces/issues
Depends: mlr3tuning (>= 0.15.0), R (>= 3.1.0)
Imports: cli, checkmate (>= 2.0.0), data.table (>= 1.14.0), mlr3 (>=
        0.11.0), mlr3misc (>= 0.11.0), paradox (>= 0.7.1), R6 (>=
        2.5.0)
Suggests: e1071 (>= 1.7-6), bbotk, glmnet (>= 4.1-2), mlr3learners (>=
        0.4.5), mlr3pipelines, ranger (>= 0.12.1), rpart (>= 4.1-15),
        testthat (>= 3.0.0), xgboost (>= 1.4.1.1)
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.2
Collate: 'mlr_tuning_spaces.R' 'TuningSpace.R' 'bibentries.R' 'sugar.R'
        'tuning_spaces_default.R' 'tuning_spaces_rbv1.R'
        'tuning_spaces_rbv2.R' 'zzz.R'
NeedsCompilation: no
Packaged: 2025-05-16 21:04:21 UTC; marc
Author: Marc Becker [cre, aut] (ORCID: <https://orcid.org/0000-0002-8115-0400>),
  Michel Lang [ctb] (ORCID: <https://orcid.org/0000-0001-9754-0393>)
Maintainer: Marc Becker <marcbecker@posteo.de>
Repository: CRAN
Date/Publication: 2025-05-16 23:40:01 UTC
Built: R 4.6.0; ; 2026-04-26 11:51:16 UTC; unix

Package: mlr3tuningspaces
Title: Search Spaces for 'mlr3'
Version: 0.7.0
Authors@R: c(
    person("Marc", "Becker", , "marcbecker@posteo.de", role = c("cre", "aut"),
           comment = c(ORCID = "0000-0002-8115-0400")),
    person("Michel", "Lang", , "michellang@gmail.com", role = "ctb",
           comment = c(ORCID = "0000-0001-9754-0393"))
  )
Description: Collection of search spaces for hyperparameter optimization in the
  'mlr3' ecosystem. It features ready-to-use search spaces for many popular
  machine learning algorithms. The search spaces are from scientific articles
  and work for a wide range of data sets.
License: LGPL-3
URL: https://mlr3tuningspaces.mlr-org.com,
        https://github.com/mlr-org/mlr3tuningspaces
BugReports: https://github.com/mlr-org/mlr3tuningspaces/issues
Depends: mlr3tuning (>= 0.15.0), R (>= 3.1.0)
Imports: cli, checkmate (>= 2.0.0), data.table (>= 1.14.0), mlr3 (>=
        0.11.0), mlr3misc (>= 0.11.0), paradox (>= 0.7.1), R6 (>=
        2.5.0)
Suggests: e1071 (>= 1.7-6), bbotk, glmnet (>= 4.1-2), mlr3learners (>=
        0.4.5), mlr3pipelines, ranger (>= 0.12.1), rpart (>= 4.1-15),
        testthat (>= 3.0.0), xgboost (>= 1.4.1.1), torch (>= 0.15.0),
        mlr3torch (>= 0.3)
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.2
Collate: 'mlr_tuning_spaces.R' 'TuningSpace.R' 'bibentries.R' 'sugar.R'
        'tuning_spaces_default.R' 'tuning_spaces_rbv1.R'
        'tuning_spaces_rbv2.R' 'tuning_spaces_rtdl.R' 'zzz.R'
NeedsCompilation: no
Packaged: 2026-07-25 11:19:19 UTC; marc
Author: Marc Becker [cre, aut] (ORCID: <https://orcid.org/0000-0002-8115-0400>),
  Michel Lang [ctb] (ORCID: <https://orcid.org/0000-0001-9754-0393>)
Maintainer: Marc Becker <marcbecker@posteo.de>
Repository: CRAN
Date/Publication: 2026-07-25 11:50:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(as.data.table,DictionaryTuningSpaces)
S3method(as.data.table,TuningSpace)
S3method(as_search_space,TuningSpace)
S3method(lts,Learner)
S3method(lts,character)
S3method(lts,missing)
S3method(rd_info,TuningSpace)
export(TuningSpace)
export(lts)
export(ltss)
export(mlr_tuning_spaces)
import(checkmate)
import(cli)
import(data.table)
import(mlr3)
import(mlr3misc)
import(mlr3tuning)
import(paradox)
importFrom(R6,R6Class)
importFrom(utils,bibentry)

# mlr3tuningspaces <img src="man/figures/logo.png" align="right" width = "120" />

Package website: [release](https://mlr3tuningspaces.mlr-org.com/) \|
[dev](https://mlr3tuningspaces.mlr-org.com/dev/)

<!-- badges: start -->

[![r-cmd-check](https://github.com/mlr-org/mlr3tuningspaces/actions/workflows/r-cmd-check.yml/badge.svg)](https://github.com/mlr-org/mlr3tuningspaces/actions/workflows/r-cmd-check.yml)
[![CRAN
Status](https://www.r-pkg.org/badges/version-ago/mlr3tuningspaces)](https://cran.r-project.org/package=mlr3tuningspaces)
[![StackOverflow](https://img.shields.io/badge/stackoverflow-mlr3-orange.svg)](https://stackoverflow.com/questions/tagged/mlr3)
[![Mattermost](https://img.shields.io/badge/chat-mattermost-orange.svg)](https://lmmisld-lmu-stats-slds.srv.mwn.de/mlr_invite/)
<!-- badges: end -->

*mlr3tuningspaces* is a collection of search spaces for hyperparameter
optimization in the [mlr3](https://github.com/mlr-org/mlr3/) ecosystem.
It features ready-to-use search spaces for many popular machine learning
algorithms. The search spaces are from scientific articles and work for
a wide range of data sets. Currently, we offer tuning spaces from four
publications.

| Publication                                        | Learner        | n Hyperparameter |
|----------------------------------------------------|----------------|-------------------|
| Bischl et al. (2023)                              | glmnet         | 2                 |
|                                                   | ranger         | 4                 |
|                                                   | rpart          | 3                 |
|                                                   | svm            | 4                 |
|                                                   | xgboost        | 8                 |
| Kuehn et al. (2018)                               | glmnet         | 2                 |
|                                                   | ranger         | 8                 |
|                                                   | rpart          | 4                 |
|                                                   | svm            | 5                 |
|                                                   | xgboost        | 13                |
| Binder, Pfisterer, and Bischl (2020)              | glmnet         | 2                 |
|                                                   | ranger         | 6                 |
|                                                   | rpart          | 4                 |
|                                                   | svm            | 4                 |
|                                                   | xgboost        | 10                |
| Gorishniy, Rubachev, Khrulkov, and Babenko (2021) | mlp            | 7                 |
|                                                   | tab_resnet     | 9                 |
|                                                   | ft_transformer | 12                |

## Resources

There are several sections about hyperparameter optimization in the
[mlr3book](https://mlr3book.mlr-org.com).

- Getting started with the
  [book](https://mlr3book.mlr-org.com/chapters/chapter4/hyperparameter_optimization.html#sec-tuning-spaces)
  section on mlr3tuningspaces.
- Learn about [search
  space](https://mlr3book.mlr-org.com/chapters/chapter4/hyperparameter_optimization.html#sec-learner-search-space).

## Installation

Install the last release from CRAN:

``` r
install.packages("mlr3tuningspaces")
```

Install the development version from GitHub:

``` r
remotes::install_github("mlr-org/mlr3tuningspaces")
```

## Example

### Quick Tuning

A learner passed to the `lts()` function arguments the learner with the
default tuning space from Bischl et al. (2023).

``` r
library(mlr3tuningspaces)

learner = lts(lrn("classif.rpart"))

# tune learner on sonar data set
instance = tune(
  tnr("random_search"),
  task = tsk("sonar"),
  learner = learner,
  resampling = rsmp("holdout"),
  measure = msr("classif.ce"),
  term_evals = 10
)

# best performing hyperparameter configuration
instance$result
```

    ##          cp minbucket minsplit learner_param_vals  x_domain classif.ce
    ## 1: -2.50293  3.110378  1.83171          <list[4]> <list[3]>  0.2148438

### Tuning Search Spaces

The `mlr_tuning_spaces` dictionary contains all tuning spaces.

``` r
library("data.table")

# print keys and tuning spaces
as.data.table(mlr_tuning_spaces)
```

A key passed to the `lts()` function returns the `TuningSpace`.

``` r
tuning_space = lts("classif.rpart.rbv2")
tuning_space
```

    ## <TuningSpace:classif.rpart.rbv2>: Classification Rpart with RandomBot
    ##           id lower upper levels logscale
    ## 1:        cp 1e-04     1 [NULL]     TRUE
    ## 2:  maxdepth 1e+00    30 [NULL]    FALSE
    ## 3: minbucket 1e+00   100 [NULL]    FALSE
    ## 4:  minsplit 1e+00   100 [NULL]    FALSE

Get the learner with tuning space.

``` r
tuning_space$get_learner()
```

    ## 
    ## ── <LearnerClassifRpart> (classif.rpart): Classification Tree ──────────────────────────────────────
    ## • Model: -
    ## • Parameters: cp=<RangeTuneToken>, maxdepth=<RangeTuneToken>, minbucket=<RangeTuneToken>,
    ## minsplit=<RangeTuneToken>, xval=0
    ## • Packages: mlr3 and rpart
    ## • Predict Types: [response] and prob
    ## • Feature Types: logical, integer, numeric, factor, and ordered
    ## • Encapsulation: none (fallback: -)
    ## • Properties: importance, missings, multiclass, selected_features, twoclass, and weights
    ## • Other settings: use_weights = 'use'

### Pipelines

Tuning spaces can be applied to the learners in a pipeline.

``` r
library(mlr3pipelines)

# set default tuning space
graph_learner = as_learner(po("subsample") %>>%
  lts(lrn("classif.rpart")))

# set rbv2 tuning space
tuning_space = lts("classif.rpart.rbv2")
graph_learner$graph$pipeops$classif.rpart$param_set$set_values(.values = tuning_space$values)
```

### Adding New Tuning Spaces

We are looking forward to new collections of tuning spaces from
peer-reviewed articles. You can suggest new tuning spaces in an issue or
contribute a new collection yourself in a pull request. Take a look at
an already implemented collection e.g. our [default tuning
spaces](https://github.com/mlr-org/mlr3tuningspaces/blob/main/R/tuning_spaces_default.R)
from Bischl et al. (2023). A `TuningSpace` is added to the
`mlr_tuning_spaces` dictionary with the `add_tuning_space()` function.
Create a tuning space for each variant of the learner e.g. for
`LearnerClassifRpart` and `LearnerRegrRpart`.

``` r
vals = list(
  minsplit  = to_tune(2, 64, logscale = TRUE),
  cp        = to_tune(1e-04, 1e-1, logscale = TRUE)
)

add_tuning_space(
  id = "classif.rpart.example",
  values = vals,
  tags = c("default", "classification"),
  learner = "classif.rpart",
  label = "Classification Tree Example"
)
```

Choose a name that is related to the publication and adjust the
documentation.

The reference is added to the `bibentries.R` file

``` r
bischl_2021 = bibentry("misc",
  key           = "bischl_2021",
  title         = "Hyperparameter Optimization: Foundations, Algorithms, Best Practices and Open Challenges",
  author        = "Bernd Bischl and Martin Binder and Michel Lang and Tobias Pielok and Jakob Richter and Stefan Coors and Janek Thomas and Theresa Ullmann and Marc Becker and Anne-Laure Boulesteix and Difan Deng and Marius Lindauer",
  year          = "2021",
  eprint        = "2107.05847",
  archivePrefix = "arXiv",
  primaryClass  = "stat.ML",
  url           = "https://arxiv.org/abs/2107.05847"
)
```

We are happy to help you with the pull request if you have any
questions.

## References

<div id="refs" class="references csl-bib-body hanging-indent">

<div id="ref-binder_2020" class="csl-entry">

Binder, Martin, Florian Pfisterer, and Bernd Bischl. 2020. “Collecting
Empirical Data about Hyperparameters for Data Driven AutoML.”
<https://www.automl.org/wp-content/uploads/2020/07/AutoML_2020_paper_63.pdf>.

</div>

<div id="ref-bischl_2021" class="csl-entry">

Bischl, Bernd, Martin Binder, Michel Lang, Tobias Pielok, Jakob Richter,
Stefan Coors, Janek Thomas, et al. 2023. “Hyperparameter Optimization:
Foundations, Algorithms, Best Practices and Open Challenges.” *Wiley
Interdisciplinary Reviews: Data Mining and Knowledge Discovery*. Wiley
Online Library.

</div>

<div id="ref-kuehn_2018" class="csl-entry">

Kuehn, Daniel, Philipp Probst, Janek Thomas, and Bernd Bischl. 2018.
“Automatic Exploration of Machine Learning Experiments on OpenML.”
<https://arxiv.org/abs/1806.10961>.

</div>

</div>

# mlr3tuningspaces <img src="man/figures/logo.png" align="right" width = "120" />

Package website: [release](https://mlr3tuningspaces.mlr-org.com/) \|
[dev](https://mlr3tuningspaces.mlr-org.com/dev/)

<!-- badges: start -->

[![r-cmd-check](https://github.com/mlr-org/mlr3tuningspaces/actions/workflows/r-cmd-check.yml/badge.svg)](https://github.com/mlr-org/mlr3tuningspaces/actions/workflows/r-cmd-check.yml)
[![CRAN
Status](https://www.r-pkg.org/badges/version-ago/mlr3tuningspaces)](https://cran.r-project.org/package=mlr3tuningspaces)
[![StackOverflow](https://img.shields.io/badge/stackoverflow-mlr3-orange.svg)](https://stackoverflow.com/questions/tagged/mlr3)
[![Mattermost](https://img.shields.io/badge/chat-mattermost-orange.svg)](https://lmmisld-lmu-stats-slds.srv.mwn.de/mlr_invite/)
<!-- badges: end -->

*mlr3tuningspaces* is a collection of search spaces for hyperparameter
optimization in the [mlr3](https://github.com/mlr-org/mlr3/) ecosystem.
It features ready-to-use search spaces for many popular machine learning
algorithms. The search spaces are from scientific articles and work for
a wide range of data sets. Currently, we offer tuning spaces from three
publications.

| Publication                          | Learner | n Hyperparameter |
|--------------------------------------|---------|------------------|
| Bischl et al. (2023)                 | glmnet  | 2                |
|                                      | ranger  | 4                |
|                                      | rpart   | 3                |
|                                      | svm     | 4                |
|                                      | xgboost | 8                |
| Kuehn et al. (2018)                  | glmnet  | 2                |
|                                      | ranger  | 8                |
|                                      | rpart   | 4                |
|                                      | svm     | 5                |
|                                      | xgboost | 13               |
| Binder, Pfisterer, and Bischl (2020) | glmnet  | 2                |
|                                      | ranger  | 6                |
|                                      | rpart   | 4                |
|                                      | svm     | 4                |
|                                      | xgboost | 10               |

## Resources

There are several sections about hyperparameter optimization in the
[mlr3book](https://mlr3book.mlr-org.com).

- Getting started with the
  [book](https://mlr3book.mlr-org.com/chapters/chapter4/hyperparameter_optimization.html#sec-tuning-spaces)
  section on mlr3tuningspaces.
- Learn about [search
  space](https://mlr3book.mlr-org.com/chapters/chapter4/hyperparameter_optimization.html#sec-learner-search-space).

## Installation

Install the last release from CRAN:

``` r
install.packages("mlr3tuningspaces")
```

Install the development version from GitHub:

``` r
remotes::install_github("mlr-org/mlr3tuningspaces")
```

## Example

### Quick Tuning

A learner passed to the `lts()` function arguments the learner with the
default tuning space from Bischl et al. (2023).

``` r
library(mlr3tuningspaces)

learner = lts(lrn("classif.rpart"))

# tune learner on pima data set
instance = tune(
  tnr("random_search"),
  task = tsk("pima"),
  learner = learner,
  resampling = rsmp("holdout"),
  measure = msr("classif.ce"),
  term_evals = 10
)

# best performing hyperparameter configuration
instance$result
```

    ##          cp minbucket minsplit learner_param_vals  x_domain classif.ce
    ## 1: -2.50293  3.110378  1.83171          <list[4]> <list[3]>  0.2148438

### Tuning Search Spaces

The `mlr_tuning_spaces` dictionary contains all tuning spaces.

``` r
library("data.table")

# print keys and tuning spaces
as.data.table(mlr_tuning_spaces)
```

A key passed to the `lts()` function returns the `TuningSpace`.

``` r
tuning_space = lts("classif.rpart.rbv2")
tuning_space
```

    ## <TuningSpace:classif.rpart.rbv2>: Classification Rpart with RandomBot
    ##           id lower upper levels logscale
    ## 1:        cp 1e-04     1 [NULL]     TRUE
    ## 2:  maxdepth 1e+00    30 [NULL]    FALSE
    ## 3: minbucket 1e+00   100 [NULL]    FALSE
    ## 4:  minsplit 1e+00   100 [NULL]    FALSE

Get the learner with tuning space.

``` r
tuning_space$get_learner()
```

    ## 
    ## ── <LearnerClassifRpart> (classif.rpart): Classification Tree ──────────────────────────────────────
    ## • Model: -
    ## • Parameters: cp=<RangeTuneToken>, maxdepth=<RangeTuneToken>, minbucket=<RangeTuneToken>,
    ## minsplit=<RangeTuneToken>, xval=0
    ## • Packages: mlr3 and rpart
    ## • Predict Types: [response] and prob
    ## • Feature Types: logical, integer, numeric, factor, and ordered
    ## • Encapsulation: none (fallback: -)
    ## • Properties: importance, missings, multiclass, selected_features, twoclass, and weights
    ## • Other settings: use_weights = 'use'

### Pipelines

Tuning spaces can be applied to the learners in a pipeline.

``` r
library(mlr3pipelines)

# set default tuning space
graph_learner = as_learner(po("subsample") %>>%
  lts(lrn("classif.rpart")))

# set rbv2 tuning space
tuning_space = lts("classif.rpart.rbv2")
graph_learner$graph$pipeops$classif.rpart$param_set$set_values(.values = tuning_space$values)
```

### Adding New Tuning Spaces

We are looking forward to new collections of tuning spaces from
peer-reviewed articles. You can suggest new tuning spaces in an issue or
contribute a new collection yourself in a pull request. Take a look at
an already implemented collection e.g. our [default tuning
spaces](https://github.com/mlr-org/mlr3tuningspaces/blob/main/R/tuning_spaces_default.R)
from Bischl et al. (2023). A `TuningSpace` is added to the
`mlr_tuning_spaces` dictionary with the `add_tuning_space()` function.
Create a tuning space for each variant of the learner e.g. for
`LearnerClassifRpart` and `LearnerRegrRpart`.

``` r
vals = list(
  minsplit  = to_tune(2, 64, logscale = TRUE),
  cp        = to_tune(1e-04, 1e-1, logscale = TRUE)
)

add_tuning_space(
  id = "classif.rpart.example",
  values = vals,
  tags = c("default", "classification"),
  learner = "classif.rpart",
  label = "Classification Tree Example"
)
```

Choose a name that is related to the publication and adjust the
documentation.

The reference is added to the `bibentries.R` file

``` r
bischl_2021 = bibentry("misc",
  key           = "bischl_2021",
  title         = "Hyperparameter Optimization: Foundations, Algorithms, Best Practices and Open Challenges",
  author        = "Bernd Bischl and Martin Binder and Michel Lang and Tobias Pielok and Jakob Richter and Stefan Coors and Janek Thomas and Theresa Ullmann and Marc Becker and Anne-Laure Boulesteix and Difan Deng and Marius Lindauer",
  year          = "2021",
  eprint        = "2107.05847",
  archivePrefix = "arXiv",
  primaryClass  = "stat.ML",
  url           = "https://arxiv.org/abs/2107.05847"
)
```

We are happy to help you with the pull request if you have any
questions.

## References

<div id="refs" class="references csl-bib-body hanging-indent">

<div id="ref-binder_2020" class="csl-entry">

Binder, Martin, Florian Pfisterer, and Bernd Bischl. 2020. “Collecting
Empirical Data about Hyperparameters for Data Driven AutoML.”
<https://www.automl.org/wp-content/uploads/2020/07/AutoML_2020_paper_63.pdf>.

</div>

<div id="ref-bischl_2021" class="csl-entry">

Bischl, Bernd, Martin Binder, Michel Lang, Tobias Pielok, Jakob Richter,
Stefan Coors, Janek Thomas, et al. 2023. “Hyperparameter Optimization:
Foundations, Algorithms, Best Practices and Open Challenges.” *Wiley
Interdisciplinary Reviews: Data Mining and Knowledge Discovery*. Wiley
Online Library.

</div>

<div id="ref-kuehn_2018" class="csl-entry">

Kuehn, Daniel, Philipp Probst, Janek Thomas, and Bernd Bischl. 2018.
“Automatic Exploration of Machine Learning Experiments on OpenML.”
<https://arxiv.org/abs/1806.10961>.

</div>

</div>

#' @title Deep Learning Tuning Spaces from Yandex's RTDL
#'
#' @name mlr_tuning_spaces_rtdl
#'
#' @description
#' Tuning spaces for deep neural network architectures from the `r cite_bib("gorishniy2021revisiting")` article.
#'
#' These tuning spaces require optimizers that have a `weight_decay` parameter, such as AdamW or any of the other optimizers built into `mlr3torch`.
#'
#' When the article suggests multiple ranges for a given hyperparameter, these tuning spaces choose the widest range.
#' 
#' The FT-Transformer tuning space disables weight decay for all bias parameters, matching the implementation provided by the authors in the rtdl-revisiting-models package. 
#' However, this differs from the experiments described in the article, which states that the
#' 
#' For the FT-Transformer, if training is unstable, consider a combination of standardizing features, using an adaptive optimizer (e.g. Adam), reducing the learning rate,
#' and using a learning rate scheduler.
#'
#' @source
#' `r format_bib("gorishniy2021revisiting")`
#'
#' @aliases
#' mlr_tuning_spaces_classif.mlp.rtdl
#' mlr_tuning_spaces_classif.tab_resnet.rtdl
#' mlr_tuning_spaces_classif.ft_transformer.rtdl
#' mlr_tuning_spaces_regr.mlp.rtdl
#' mlr_tuning_spaces_regr.tab_resnet.rtdl
#' mlr_tuning_spaces_regr.ft_transformer.rtdl
#' 
#' @section MLP tuning space:
#' `r rd_info(lts("classif.mlp.rtdl"))`
#' 
#' @section Tabular ResNet tuning space:
#' `r rd_info(lts("classif.tab_resnet.rtdl"))`
#' 
#' @section FT-Transformer tuning space:
#' `r rd_info(lts("classif.ft_transformer.rtdl"))`
#' 
#' In the FT-Transformer, the validation-related parameters must still be set manually, via e.g. `lts("regr.ft_transformer.rtdl")$get_learner(validate = 0.2, measures_valid = msr("regr.rmse"))`.
#'
#' @include mlr_tuning_spaces.R
NULL

# mlp
vals = list(
  n_layers          = to_tune(1, 16),
  neurons           = to_tune(levels = 1:1024),
  p                 = to_tune(0, 0.5),
  opt.lr            = to_tune(1e-5, 1e-2, logscale = TRUE),
  opt.weight_decay  = to_tune(1e-6, 1e-3, logscale = TRUE),
  epochs            = to_tune(lower = 1L, upper = 100L, internal = TRUE),
  patience          = 17L
)

add_tuning_space(
  id = "classif.mlp.rtdl",
  values = vals,
  tags = c("gorishniy2021", "classification"),
  learner = "classif.mlp",
  package = "mlr3torch",
  label = "Classification MLP with RTDL"
)

add_tuning_space(
  id = "regr.mlp.rtdl",
  values = vals,
  tags = c("gorishniy2021", "regression"),
  learner = "regr.mlp",
  package = "mlr3torch",
  label = "Regression MLP with RTDL"
)

# resnet
vals = list(
  n_blocks            = to_tune(1, 16),
  d_block             = to_tune(64, 1024),
  d_hidden_multiplier = to_tune(1, 4),
  dropout1            = to_tune(0, 0.5),
  dropout2            = to_tune(0, 0.5),
  opt.lr              = to_tune(1e-5, 1e-2, logscale = TRUE),
  opt.weight_decay    = to_tune(1e-6, 1e-3, logscale = TRUE),
  epochs              = to_tune(lower = 1L, upper = 100L, internal = TRUE),
  patience            = 17L
)

add_tuning_space(
  id = "classif.tab_resnet.rtdl",
  values = vals,
  tags = c("gorishniy2021", "classification"),
  learner = "classif.tab_resnet",
  package = "mlr3torch",
  label = "Classification Tabular ResNet with RTDL"
)

add_tuning_space(
  id = "regr.tab_resnet.rtdl",
  values = vals,
  tags = c("gorishniy2021", "regression"),
  learner = "regr.tab_resnet",
  package = "mlr3torch",
  label = "Regression Tabular ResNet with RTDL"
)

no_wd = function(name) {
  # this will also disable weight decay for the input projection bias of the attention heads
  no_wd_params = c("_normalization", "bias")

  return(any(map_lgl(no_wd_params, function(pattern) grepl(pattern, name, fixed = TRUE))))
}

rtdl_param_groups = function(parameters) {
  split_param_names = strsplit(names(parameters), ".", fixed = TRUE)

  ffn_norm_idx = grepl("ffn_normalization", names(parameters), fixed = TRUE)
  first_ffn_norm_num_in_module_list = as.integer(split_param_names[ffn_norm_idx][[1]][2])
  cls_num_in_module_list = first_ffn_norm_num_in_module_list - 1
  nums_in_module_list = sapply(split_param_names, function(x) as.integer(x[2]))
  tokenizer_idx = nums_in_module_list < cls_num_in_module_list

  # the last normalization layer is unnamed, so we need to find it based on its position in the module list
  last_module_num_in_module_list = as.integer(split_param_names[[length(split_param_names)]][2])
  last_norm_num_in_module_list = last_module_num_in_module_list - 2
  last_norm_idx = nums_in_module_list == last_norm_num_in_module_list

  no_wd_idx = map_lgl(names(parameters), no_wd) | tokenizer_idx | last_norm_idx
  no_wd_group = parameters[no_wd_idx]

  main_group = parameters[!no_wd_idx]

  list(
    list(params = main_group),
    list(params = no_wd_group, weight_decay = 0)
  )
}

# ft_transformer
vals = list(
  n_blocks                = to_tune(1, 6),
  d_token                 = to_tune(p_int(8L, 64L, trafo = function(x) 8L * x)),
  attention_n_heads       = 8L,
  residual_dropout        = to_tune(0, 0.2),
  attention_dropout       = to_tune(0, 0.5),
  ffn_dropout             = to_tune(0, 0.5),
  ffn_d_hidden_multiplier = to_tune(2 / 3, 8 / 3),
  opt.lr                  = to_tune(1e-5, 1e-4, logscale = TRUE),
  opt.weight_decay        = to_tune(1e-6, 1e-3, logscale = TRUE),
  opt.param_groups        = rtdl_param_groups,
  epochs                  = to_tune(lower = 1L, upper = 100L, internal = TRUE),
  patience                = 17L
)

add_tuning_space(
  id = "classif.ft_transformer.rtdl",
  values = vals,
  tags = c("gorishniy2021", "classification"),
  learner = "classif.ft_transformer",
  package = "mlr3torch",
  label = "Classification FT-Transformer with RTDL"
)

add_tuning_space(
  id = "regr.ft_transformer.rtdl",
  values = vals,
  tags = c("gorishniy2021", "regression"),
  learner = "regr.ft_transformer",
  package = "mlr3torch",
  label = "Regression FT-Transformer with RTDL"
)

#' @title RandomBot V2 Tuning Spaces
#'
#' @name mlr_tuning_spaces_rbv2
#'
#' @description
#' Tuning spaces from the `r cite_bib("binder_2020")` article.
#'
#' @source
#' `r format_bib("binder_2020")`
#'
#' @aliases
#' mlr_tuning_spaces_classif.glmnet.rbv2
#' mlr_tuning_spaces_classif.ranger.rbv2
#' mlr_tuning_spaces_classif.rpart.rbv2
#' mlr_tuning_spaces_classif.svm.rbv2
#' mlr_tuning_spaces_classif.xgboost.rbv2
#' mlr_tuning_spaces_regr.glmnet.rbv2
#' mlr_tuning_spaces_regr.ranger.rbv2
#' mlr_tuning_spaces_regr.rpart.rbv2
#' mlr_tuning_spaces_regr.svm.rbv2
#' mlr_tuning_spaces_regr.xgboost.rbv2
#'
#' @section glmnet tuning space:
#' `r rd_info(lts("classif.glmnet.rbv2"))`
#'
#' @section ranger tuning space:
#' `r rd_info(lts("classif.ranger.rbv2"))`
#'
#' `mtry.power` is replaced by `mtry.ratio`.
#'
#' @section rpart tuning space:
#' `r rd_info(lts("classif.rpart.rbv2"))`
#'
#' @section svm tuning space:
#' `r rd_info(lts("classif.svm.rbv2"))`
#'
#' @section xgboost tuning space:
#' `r rd_info(lts("classif.xgboost.rbv2"))`
#'
#' @include mlr_tuning_spaces.R
NULL

# glmnet
vals = list(
  alpha = to_tune(0, 1),
  s     = to_tune(1e-4, 1e3, logscale = TRUE)
)

add_tuning_space(
  id = "classif.glmnet.rbv2",
  values = vals,
  tags = c("rbv2", "classification"),
  learner = "classif.glmnet",
  package = "mlr3learners",
  label = "Classification GLM with RandomBot"
)

add_tuning_space(
  id = "regr.glmnet.rbv2",
  values = vals,
  tags = c("rbv2", "regression"),
  learner = "regr.glmnet",
  package = "mlr3learners",
  label = "Regression GLM with RandomBot"
)

# ranger
vals = list(
  num.trees                 = to_tune(1, 2000),
  replace                   = to_tune(p_lgl()),
  sample.fraction           = to_tune(0.1, 1),
  mtry.ratio                = to_tune(0, 1),
  respect.unordered.factors = to_tune(c("ignore", "order", "partition")),
  min.node.size             = to_tune(p_int(1, 100)),
  splitrule                 = to_tune(c("gini", "extratrees")),
  num.random.splits         = to_tune(1, 100)
)

add_tuning_space(
  id = "classif.ranger.rbv2",
  values = vals,
  tags = c("rbv2", "classification"),
  learner = "classif.ranger",
  package = "mlr3learners",
  label = "Classification Ranger with RandomBot"
)

vals = list(
  num.trees                 = to_tune(1, 2000),
  replace                   = to_tune(p_lgl()),
  sample.fraction           = to_tune(0.1, 1),
  mtry.ratio                = to_tune(0, 1),
  respect.unordered.factors = to_tune(c("ignore", "order", "partition")),
  min.node.size             = to_tune(p_int(1, 100)),
  num.random.splits         = to_tune(1, 100)
)

add_tuning_space(
  id = "regr.ranger.rbv2",
  values = vals,
  tags = c("rbv2", "regression"),
  learner = "regr.ranger",
  package = "mlr3learners",
  label = "Regression Ranger with RandomBot"
)

# rpart
vals = list(
  cp        = to_tune(1e-4, 1, logscale = TRUE),
  maxdepth  = to_tune(1, 30),
  minbucket = to_tune(1, 100),
  minsplit  = to_tune(1, 100)
)

add_tuning_space(
  id = "classif.rpart.rbv2",
  values = vals,
  tags = c("rbv2", "classification"),
  learner = "classif.rpart",
  package = "mlr3learners",
  label = "Classification Rpart with RandomBot"
)

add_tuning_space(
  id = "regr.rpart.rbv2",
  values = vals,
  tags = c("rbv2", "regression"),
  learner = "regr.rpart",
  package = "mlr3learners",
  label = "Regression Rpart with RandomBot"
)

# svm
vals = list(
  kernel    = to_tune(c("linear", "polynomial", "radial")),
  cost      = to_tune(1e-4, 1e3, logscale = TRUE),
  gamma     = to_tune(1e-4, 1e3, logscale = TRUE),
  tolerance = to_tune(1e-4, 2, logscale = TRUE),
  degree    = to_tune(2, 5)
)

add_tuning_space(
  id = "classif.svm.rbv2",
  values = vals,
  tags = c("rbv2", "classification"),
  learner = "classif.svm",
  package = "mlr3learners",
  label = "Classification SVM with RandomBot"
)

add_tuning_space(
  id = "regr.svm.rbv2",
  values = vals,
  tags = c("rbv2", "regression"),
  learner = "regr.svm",
  package = "mlr3learners",
  label = "Regression SVM with RandomBot"
)

# xgboost
vals = list(
  booster           = to_tune(c("gblinear", "gbtree", "dart")),
  nrounds           = to_tune(7, 2981, logscale = TRUE),
  eta               = to_tune(1e-4, 1, logscale = TRUE),
  gamma             = to_tune(1e-5, 7, logscale = TRUE),
  lambda            = to_tune(1e-4, 1e3, logscale = TRUE),
  alpha             = to_tune(1e-4, 1e3, logscale = TRUE),
  subsample         = to_tune(0.1, 1),
  max_depth         = to_tune(1, 15),
  min_child_weight  = to_tune(1, 1e2, logscale = TRUE),
  colsample_bytree  = to_tune(0.01, 1),
  colsample_bylevel = to_tune(0.01, 1),
  rate_drop         = to_tune(0, 1),
  skip_drop         = to_tune(0, 1)
)

add_tuning_space(
  id = "classif.xgboost.rbv2",
  values = vals,
  tags = c("rbv2", "classification"),
  learner = "classif.xgboost",
  package = "mlr3learners",
  label = "Classification XGBoost with RandomBot"
)

add_tuning_space(
  id = "regr.xgboost.rbv2",
  values = vals,
  tags = c("rbv2", "regression"),
  learner = "regr.xgboost",
  package = "mlr3learners",
  label = "Regression XGBoost with RandomBot"
)

#' @importFrom utils bibentry
bibentries = c(
  bischl_2021 = bibentry("misc",
    key           = "bischl_2021",
    title         = "Hyperparameter Optimization: Foundations, Algorithms, Best Practices and Open Challenges",
    author        = "Bernd Bischl and Martin Binder and Michel Lang and Tobias Pielok and Jakob Richter and Stefan Coors and Janek Thomas and Theresa Ullmann and Marc Becker and Anne-Laure Boulesteix and Difan Deng and Marius Lindauer",
    journal       = "Wiley Interdisciplinary Reviews: Data Mining and Knowledge Discovery",
    volume        = "13",
    number        = "2",
    pages         = "e1484",
    year          = "2023",
    publisher     = "Wiley Online Library"
  ),

  kuehn_2018 = bibentry("misc",
    key           = "kuehn_2018",
    title         = "Automatic Exploration of Machine Learning Experiments on OpenML",
    author        = "Daniel Kuehn and Philipp Probst and Janek Thomas and Bernd Bischl",
    year          = "2018",
    eprint        = "1806.10961",
    archivePrefix = "arXiv",
    primaryClass  = "stat.ML",
    url = "https://arxiv.org/abs/1806.10961"
  ),

  binder_2020 = bibentry("misc",
    key       = "binder_2020",
    title     = "Collecting Empirical Data About Hyperparameters for Data Driven AutoML",
    author    = "Martin Binder and Florian Pfisterer and Bernd Bischl",
    year      = "2020",
    booktitle = "Proceedings of the 7th ICML Workshop on Automated Machine Learning (AutoML 2020)",
    date      = "2020",
    url       = "https://www.automl.org/wp-content/uploads/2020/07/AutoML_2020_paper_63.pdf"
  ),

  gorishniy2021revisiting = bibentry("article",
    title = "Revisiting Deep Learning  for Tabular Data",
    author = "Yury Gorishniy and Ivan Rubachev and Valentin Khrulkov and Artem Babenko",
    journal = "arXiv",
    volume = "2106.11959",
    year = "2021",
  )
)

#' @title Syntactic Sugar for Tuning Space Construction
#'
#' @description
#' Function to retrieve [TuningSpace] objects from [mlr_tuning_spaces] and further, allows a [mlr3::Learner] to be directly configured with a search space.
#' This function belongs to [mlr3::mlr_sugar] family.
#'
#' @param x (`character()` | [mlr3::Learner])\cr
#'   If `character`, key passed the dictionary to retrieve the tuning space.
#'   If [mlr3::Learner], default tuning space is added to the learner.
#' @param ... (named list of [paradox::TuneToken] | `NULL`)\cr
#'   Pass [paradox::TuneToken] to add or overwrite parameters in the tuning space.
#'   Use `NULL` to remove parameters (see examples).
#'
#' @return
#' [TuningSpace] if `x` is `character()`.
#' [mlr3::Learner] if `x` is [mlr3::Learner].
#' Or a list of objects for the `ltss()` function.
#'
#' @export
#' @examples
#' # load tuning space
#' lts("classif.rpart.default")
#'
#' # load tuning space and add parameter
#' lts("classif.rpart.default", maxdepth = to_tune(1, 15))
#'
#' # load tuning space and remove parameter
#' lts("classif.rpart.default", minsplit = NULL)
#'
#' # load tuning space and overwrite parameter
#' lts("classif.rpart.default", minsplit = to_tune(32, 128))
#'
#' # load learner and apply tuning space in one go
#' lts(lrn("classif.rpart"))
#'
#' # load learner, overwrite parameter and apply tuning space
#' lts(lrn("classif.rpart"), minsplit = to_tune(32, 128))
#'
#' # load multiple tuning spaces
#' ltss(c("classif.rpart.default", "classif.ranger.default"))
lts = function(x, ...) {
  if (missing(x)) return(lts.missing(x))
  UseMethod("lts")
}

#' @rdname lts
#' @return missing, [mlr_tuning_spaces] dictionary
#' @export
lts.missing = function(x, ...) {
  dictionary_sugar_get(mlr_tuning_spaces, x)
}

#' @rdname lts
#' @return a `character`, [TuningSpace]
#' @export
lts.character = function(x, ...) {
  tuning_space = dictionary_sugar_get(mlr_tuning_spaces, x)
  if (!...length()) return(tuning_space)
  dots = assert_list(list(...), types = c("TuneToken", "NULL"), names = "unique")
  tuning_space$values = insert_named(tuning_space$values, dots)
  tuning_space$values = discard(tuning_space$values, is.null)
  tuning_space
}

#' @rdname lts
#' @return a [mlr3::Learner], [mlr3::Learner] with [paradox::TuneToken]
#' @export
lts.Learner = function(x, ...) {
  tuning_space = lts(paste0(x$id, ".default"), ...)
  x$param_set$values = insert_named(x$param_set$values, tuning_space$values)
  x
}

#' @rdname lts
#' @return a `list()`, list of [TuningSpace] or [mlr3::Learner]
#' @export
ltss = function(x) {
  map(x, function(key) {
    lts(key)
  })
}

test_that("mlr_tuning_spaces", {
  expect_dictionary(mlr_tuning_spaces, min_items = 1L)
  keys = mlr_tuning_spaces$keys()

  for (key in keys) {
    tuning_space = lts(key)
    expect_r6(tuning_space, "TuningSpace")
  }
})

test_that("lts character dispatch works", {
  tuning_space = lts("classif.rpart.default")
  expect_r6(tuning_space, "TuningSpace")
})

test_that("lts learner dispatch works", {
  learner = lts(lrn("classif.rpart"))
  expect_learner(learner)
})

test_that("lts learner dispatch with constants works", {
  learner = lts(lrn("classif.rpart", xval = 1))
  expect_learner(learner)
  expect_equal(learner$param_set$values$xval, 1)
  expect_class(learner$param_set$values$minsplit, "RangeTuneToken")
})

test_that("ltss character dispatch works", {
  tuning_spaces = ltss(c("classif.rpart.default", "classif.svm.default"))
  expect_list(tuning_spaces, len = 2)
  expect_r6(tuning_spaces[[1]], "TuningSpace")
  expect_r6(tuning_spaces[[2]], "TuningSpace")
})

test_that("ltss learner dispatch works", {
  learners = ltss(list(lrn("classif.rpart"), lrn("classif.svm")))
  expect_learner(learners[[1]])
  expect_learner(learners[[2]])
})

test_that("ltss learner dispatch with constants works", {
  learners = ltss(list(lrn("classif.rpart", xval = 1), lrn("classif.svm", type = "C-classification")))
  expect_learner(learners[[1]])
  expect_learner(learners[[2]])
  expect_equal(learners[[1]]$param_set$values$xval, 1)
  expect_class(learners[[1]]$param_set$values$minsplit, "RangeTuneToken")
  expect_equal(learners[[2]]$param_set$values$type, "C-classification")
  expect_class(learners[[2]]$param_set$values$degree, "RangeTuneToken")
})

test_that("as.data.table objects parameter", {
  tab = as.data.table(mlr_tuning_spaces, objects = TRUE)
  expect_data_table(tab)
  expect_list(tab$object, "TuningSpace", any.missing = FALSE)
})

test_that("get_learner function works", {
  learner = lts("classif.rpart.default")$get_learner(predict_type = "prob")
  expect_learner(learner)
  expect_equal(learner$predict_type, "prob")
})

test_that("get_learner function works with added parameters", {
  search_space = lts("classif.xgboost.default", min_child_weight = to_tune(0, 10))

  expect_class(search_space$get_learner()$param_set$values$min_child_weight, "TuneToken")
})

test_that("get_learner function works with changed parameters", {
  search_space = lts("classif.xgboost.default", nrounds = to_tune(1, 10000))

  expect_equal(search_space$get_learner()$param_set$values$nrounds$content$upper, 10000)
})

test_that("as.data.table.TuningSpace works", {
  keys =  mlr_tuning_spaces$keys()
  walk(keys, function(key) {
    tab = as.data.table(lts(key))
    expect_data_table(tab)
    expect_names(names(tab), must.include = c("id"))
  })
})

test_that("lts dispatch works", {
  # missing dispatch
  dict = lts()
  expect_class(dict, "DictionaryTuningSpaces")

  # character dispatch
  tuning_space = lts("classif.rpart.default")
  expect_r6(tuning_space, "TuningSpace")

  # learner dispatch
  learner = lts(lrn("classif.rpart"))
  expect_learner(learner)
})

test_that("ltss sugar function works", {
  # character dispatch
  tuning_spaces = ltss(c("classif.rpart.default", "classif.svm.default"))
  expect_list(tuning_spaces, len = 2)
  expect_r6(tuning_spaces[[1]], "TuningSpace")
  expect_r6(tuning_spaces[[2]], "TuningSpace")

  # learner dispatch
  learners = ltss(list(lrn("classif.rpart"), lrn("classif.svm")))
  expect_learner(learners[[1]])
  expect_learner(learners[[2]])
})

test_that("constants can be set", {
  learner = lts(lrn("classif.rpart", xval = 1))
  expect_learner(learner)
  expect_equal(learner$param_set$values$xval, 1)
  expect_class(learner$param_set$values$minsplit, "RangeTuneToken")

  learners = ltss(list(lrn("classif.rpart", xval = 1), lrn("classif.svm", type = "C-classification")))
  expect_learner(learners[[1]])
  expect_learner(learners[[2]])
  expect_equal(learners[[1]]$param_set$values$xval, 1)
  expect_class(learners[[1]]$param_set$values$minsplit, "RangeTuneToken")
  expect_equal(learners[[2]]$param_set$values$type, "C-classification")
  expect_class(learners[[2]]$param_set$values$degree, "RangeTuneToken")
})

test_that("modifying tuning space works", {
  # add
  tuning_space = lts("classif.rpart.default", maxdepth = to_tune())
  expect_r6(tuning_space, "TuningSpace")
  expect_names(names(tuning_space$values), permutation.of = c("minsplit", "minbucket", "cp", "maxdepth"))
  expect_list(tuning_space$values, types = "TuneToken")

  # remove
  tuning_space = lts("classif.rpart.default", minsplit = NULL)
  expect_r6(tuning_space, "TuningSpace")
  expect_names(names(tuning_space$values), permutation.of = c("minbucket", "cp"))
  expect_list(tuning_space$values, types = "TuneToken")

  # overwrite
  tuning_space = lts("classif.rpart.default", minsplit = to_tune(64, 128))
  expect_r6(tuning_space, "TuningSpace")
  expect_names(names(tuning_space$values), permutation.of = c("minsplit", "minbucket", "cp"))
  expect_list(tuning_space$values, types = "TuneToken")
  expect_equal(tuning_space$values$minsplit$content$lower, 64)
})

test_that("as_search_space on TuningSpace works", {
  tuning_space = lts("classif.rpart.default")
  search_space = as_search_space(tuning_space)
  expect_r6(search_space, "ParamSet")
  expect_set_equal(search_space$ids(), c("minsplit", "minbucket", "cp"))
})

test_that("search space from TuningSpace works", {
  tuning_space = lts("classif.rpart.default")
  instance = ti(task = tsk("iris"), learner = lrn("classif.rpart"),
    resampling = rsmp("holdout"), measures = msr("classif.ce"),
    terminator = trm("evals", n_evals = 1), search_space = tuning_space)

  expect_r6(instance$search_space, "ParamSet")
  expect_set_equal(instance$search_space$ids(), c("minsplit", "minbucket", "cp"))
})

test_that("search space from TuningSpace works", {
  tuning_space = lts("classif.rpart.default")
  instance = ti(task = tsk("iris"), learner = lrn("classif.rpart"),
    resampling = rsmp("holdout"), measures = msrs(c("classif.ce", "classif.acc")),
    terminator = trm("evals", n_evals = 1), search_space = tuning_space)

  expect_r6(instance$search_space, "ParamSet")
  expect_set_equal(instance$search_space$ids(), c("minsplit", "minbucket", "cp"))
})

test_that("search space from TuningSpace works", {
  tuning_space = lts("classif.rpart.default")
  at = AutoTuner$new(learner = lrn("classif.rpart"), resampling = rsmp("holdout"),
    measure = msr("classif.ce"), terminator = trm("evals", n_evals = 1),
    search_space = tuning_space, tuner = tnr("random_search"))

  expect_r6(at$instance_args$search_space, "ParamSet")
  expect_set_equal(at$instance_args$search_space$ids(), c("minsplit", "minbucket", "cp"))
})

test_that("as_search_space works with added parameters", {
  search_space = lts("classif.xgboost.default", min_child_weight = to_tune(0, 10))

  expect_names(as_search_space(search_space)$ids(), must.include = "min_child_weight")
})

test_that("as_search_space works with changed parameters", {
  search_space = lts("classif.xgboost.default", nrounds = to_tune(1, 10000))

  expect_equal(as_search_space(search_space)$upper[["nrounds"]], 10000)
})

#' @title RandomBot Tuning Spaces
#'
#' @name mlr_tuning_spaces_rbv1
#'
#' @description
#' Tuning spaces from the `r cite_bib("kuehn_2018")` article.
#'
#' @source
#' `r format_bib("kuehn_2018")`
#'
#' @aliases
#' mlr_tuning_spaces_classif.glmnet.rbv1
#' mlr_tuning_spaces_classif.ranger.rbv1
#' mlr_tuning_spaces_classif.rpart.rbv1
#' mlr_tuning_spaces_classif.svm.rbv1
#' mlr_tuning_spaces_classif.xgboost.rbv1
#' mlr_tuning_spaces_regr.glmnet.rbv1
#' mlr_tuning_spaces_regr.ranger.rbv1
#' mlr_tuning_spaces_regr.rpart.rbv1
#' mlr_tuning_spaces_regr.svm.rbv1
#' mlr_tuning_spaces_regr.xgboost.rbv1
#'
#' @section glmnet tuning space:
#' `r rd_info(lts("classif.glmnet.rbv1"))`
#'
#' @section ranger tuning space:
#' `r rd_info(lts("classif.ranger.rbv1"))`
#'
#' The tuning space of the ranger learner is slightly different from the original paper.
#' The hyperparameter `mtry.power` is replaced by `mtry.ratio` and `min.node.size` is explored in a range from 1 to 100.
#'
#' @section rpart tuning space:
#' `r rd_info(lts("classif.rpart.rbv1"))`
#'
#' @section svm tuning space:
#' `r rd_info(lts("classif.svm.rbv1"))`
#'
#' @section xgboost tuning space:
#' `r rd_info(lts("classif.xgboost.rbv1"))`
#'
#' @include mlr_tuning_spaces.R
NULL

# glmnet
vals = list(
  alpha = to_tune(0, 1),
  s     = to_tune(1e-4, 1e3, logscale = TRUE)
)

add_tuning_space(
  id = "classif.glmnet.rbv1",
  values = vals,
  tags = c("rbv1", "classification"),
  learner = "classif.glmnet",
  package = "mlr3learners",
  label = "Classification GLM with RandomBot"
)

add_tuning_space(
  id = "regr.glmnet.rbv1",
  values = vals,
  tags = c("rbv1", "regression"),
  learner = "regr.glmnet",
  package = "mlr3learners",
  label = "Regression GLM with RandomBot"
)

# ranger
vals = list(
  num.trees                 = to_tune(1, 2000),
  replace                   = to_tune(p_lgl()),
  sample.fraction           = to_tune(0.1, 1),
  mtry.ratio                = to_tune(0, 1),
  respect.unordered.factors = to_tune(c("ignore", "order")),
  min.node.size             = to_tune(p_int(1, 100))
)

add_tuning_space(
  id = "classif.ranger.rbv1",
  values = vals,
  tags = c("rbv1", "classification"),
  learner = "classif.ranger",
  package = "mlr3learners",
  label = "Classification Ranger with RandomBot"
)

vals = list(
  num.trees                 = to_tune(1, 2000),
  replace                   = to_tune(p_lgl()),
  sample.fraction           = to_tune(0.1, 1),
  mtry.ratio                = to_tune(0, 1),
  respect.unordered.factors = to_tune(c("ignore", "order")),
  min.node.size             = to_tune(p_int(1, 100))
)

add_tuning_space(
  id = "regr.ranger.rbv1",
  values = vals,
  tags = c("rbv1", "regression"),
  learner = "regr.ranger",
  package = "mlr3learners",
  label = "Regression Ranger with RandomBot"
)

# rpart
vals = list(
  cp        = to_tune(0, 1),
  maxdepth  = to_tune(1, 30),
  minbucket = to_tune(1, 60),
  minsplit  = to_tune(1, 60)
)

add_tuning_space(
  id = "classif.rpart.rbv1",
  values = vals,
  tags = c("rbv1", "classification"),
  learner = "classif.rpart",
  package = "mlr3learners",
  label = "Classification Rpart with RandomBot"
)

add_tuning_space(
  id = "regr.rpart.rbv1",
  values = vals,
  tags = c("rbv1", "regression"),
  learner = "regr.rpart",
  package = "mlr3learners",
  label = "Regression Rpart with RandomBot"
)

# svm
vals = list(
  kernel    = to_tune(c("linear", "polynomial", "radial")),
  cost      =  to_tune(1e-4, 1e3, logscale = TRUE),
  gamma     = to_tune(1e-4, 1e3, logscale = TRUE),
  degree    = to_tune(2, 5)
)

add_tuning_space(
  id = "classif.svm.rbv1",
  values = vals,
  tags = c("rbv1", "classification"),
  learner = "classif.svm",
  package = "mlr3learners",
  label = "Classification SVM with RandomBot"
)

add_tuning_space(
  id = "regr.svm.rbv1",
  values = vals,
  tags = c("rbv1", "regression"),
  learner = "regr.svm",
  package = "mlr3learners",
  label = "Regression SVM with RandomBot"
)

# xgboost
vals = list(
  nrounds           = to_tune(1, 5000),
  eta               = to_tune(1e-4, 1, logscale = TRUE),
  subsample         = to_tune(0, 1),
  booster           = to_tune(c("gblinear", "gbtree", "dart")),
  max_depth         = to_tune(1, 15),
  min_child_weight  = to_tune(1, 1e2, logscale = TRUE),
  colsample_bytree  = to_tune(0, 1),
  colsample_bylevel = to_tune(0, 1),
  lambda            = to_tune(1e-4, 1e3, logscale = TRUE),
  alpha             = to_tune(1e-4, 1e3, logscale = TRUE)
)

add_tuning_space(
  id = "classif.xgboost.rbv1",
  values = vals,
  tags = c("rbv1", "classification"),
  learner = "classif.xgboost",
  package = "mlr3learners",
  label = "Classification XGBoost with RandomBot"
)

add_tuning_space(
  id = "regr.xgboost.rbv1",
  values = vals,
  tags = c("rbv1", "regression"),
  learner = "regr.xgboost",
  package = "mlr3learners",
  label = "Regression XGBoost with RandomBot"
)

#' @title Default Tuning Spaces
#'
#' @name mlr_tuning_spaces_default
#'
#' @description
#' Tuning spaces from the `r cite_bib("bischl_2021")` article.
#'
#' @source
#' `r format_bib("bischl_2021")`
#'
#' @aliases
#' mlr_tuning_spaces_classif.glmnet.default
#' mlr_tuning_spaces_classif.ranger.default
#' mlr_tuning_spaces_classif.rpart.default
#' mlr_tuning_spaces_classif.svm.default
#' mlr_tuning_spaces_classif.xgboost.default
#' mlr_tuning_spaces_regr.glmnet.default
#' mlr_tuning_spaces_regr.ranger.default
#' mlr_tuning_spaces_regr.rpart.default
#' mlr_tuning_spaces_regr.svm.default
#' mlr_tuning_spaces_regr.xgboost.default
#'
#' @section glmnet tuning space:
#' `r rd_info(lts("classif.glmnet.default"))`
#'
#' @section ranger tuning space:
#' `r rd_info(lts("classif.ranger.default"))`
#'
#' @section rpart tuning space:
#' `r rd_info(lts("classif.rpart.default"))`
#'
#' @section svm tuning space:
#' `r rd_info(lts("classif.svm.default"))`
#'
#' @section xgboost tuning space:
#' `r rd_info(lts("classif.xgboost.default"))`
#'
#' @include mlr_tuning_spaces.R
NULL

# glmnet
vals = list(
  s     = to_tune(1e-4, 1e4, logscale = TRUE),
  alpha = to_tune(0, 1)
)

add_tuning_space(
  id = "classif.glmnet.default",
  values = vals,
  tags = c("default", "classification"),
  learner = "classif.glmnet",
  package = "mlr3learners",
  label = "Classification GLM with Default"
)

add_tuning_space(
  id = "regr.glmnet.default",
  values = vals,
  tags = c("default", "regression"),
  learner = "regr.glmnet",
  package = "mlr3learners",
  label = "Regression GLM with Default"
)

# ranger
vals = list(
  mtry.ratio      = to_tune(0, 1),
  replace         = to_tune(p_lgl()),
  sample.fraction = to_tune(1e-1, 1),
  num.trees       = to_tune(1, 2000)
)

add_tuning_space(
  id = "classif.ranger.default",
  values = vals,
  tags = c("default", "classification"),
  learner = "classif.ranger",
  label = "Classification Ranger with Default"
)

add_tuning_space(
  id = "regr.ranger.default",
  values = vals,
  tags = c("default", "regression"),
  learner = "regr.ranger",
  label = "Regression Ranger with Default"
)

# rpart
vals = list(
  minsplit  = to_tune(2, 128, logscale = TRUE),
  minbucket = to_tune(1, 64, logscale = TRUE),
  cp        = to_tune(1e-04, 1e-1, logscale = TRUE)
)

add_tuning_space(
  id = "classif.rpart.default",
  values = vals,
  tags = c("default", "classification"),
  learner = "classif.rpart",
  label = "Classification Rpart with Default"
)

add_tuning_space(
  id = "regr.rpart.default",
  values = vals,
  tags = c("default", "regression"),
  learner = "regr.rpart",
  label = "Regression Rpart with Default"
)

# svm
vals = list(
  cost    = to_tune(1e-4, 1e4, logscale = TRUE),
  kernel  = to_tune(c("polynomial", "radial", "sigmoid", "linear")),
  degree  = to_tune(2, 5),
  gamma   = to_tune(1e-4, 1e4, logscale = TRUE)
)

add_tuning_space(
  id = "classif.svm.default",
  values = vals,
  tags = c("default", "classification"),
  learner = "classif.svm",
  package = "mlr3learners",
  label = "Classification SVM with Default"
)

add_tuning_space(
  id = "regr.svm.default",
  values = vals,
  tags = c("default", "regression"),
  learner = "regr.svm",
  package = "mlr3learners",
  label = "Regression SVM with Default"
)

# xgboost
vals = list(
  eta               = to_tune(1e-4, 1, logscale = TRUE),
  nrounds           = to_tune(1, 5000),
  max_depth         = to_tune(1, 20),
  colsample_bytree  = to_tune(1e-1, 1),
  colsample_bylevel = to_tune(1e-1, 1),
  lambda            = to_tune(1e-3, 1e3, logscale = TRUE),
  alpha             = to_tune(1e-3, 1e3, logscale = TRUE),
  subsample         = to_tune(1e-1, 1)
)

add_tuning_space(
  id = "classif.xgboost.default",
  values = vals,
  tags = c("default", "classification"),
  learner = "classif.xgboost",
  package = "mlr3learners",
  label = "Classification XGBoost with Default"
)

add_tuning_space(
  id = "regr.xgboost.default",
  values = vals,
  tags = c("default", "regression"),
  learner = "regr.xgboost",
  package = "mlr3learners",
  label = "Regression XGBoost with Default"
)

#' @title Tuning Spaces
#'
#' @description
#' This class defines a tuning space for hyperparameter tuning.
#'
#' For tuning, it is important to create a search space that defines the range over which hyperparameters should be tuned.
#' `TuningSpace` object consists of search spaces from peer-reviewed articles which work well for a wide range of data sets.
#'
#' The `$values` field stores a list of [paradox::TuneToken] which define the search space.
#' These tokens can be assigned to the `$values` slot of a learner's [paradox::ParamSet].
#' When the learner is tuned, the tokens are used to create the search space.
#'
#' @section S3 Methods:
#' * `as.data.table.TuningSpace(x)`\cr
#' Returns a tabular view of the tuning space.\cr
#' [TuningSpace] -> [data.table::data.table()]\cr
#'     * `x` ([TuningSpace])
#'
#' @export
#' @examples
#' library(mlr3tuning)
#'
#' # Get default tuning space of rpart learner
#' tuning_space = lts("classif.rpart.default")
#'
#' # Set tuning space
#' learner = lrn("classif.rpart")
#' learner$param_set$values = tuning_space$values
#'
#' # Tune learner
#' instance = tune(
#'   tnr("random_search"),
#'   task = tsk("sonar"),
#'   learner = learner,
#'   resampling = rsmp ("holdout"),
#'   measure = msr("classif.ce"),
#'   term_evals = 10)
#'
#' instance$result
#'
#' library(mlr3pipelines)
#'
#' # Set tuning space in a pipeline
#' graph_learner = as_learner(po("subsample") %>>%
#'   lts(lrn("classif.rpart")))
TuningSpace = R6Class("TuningSpace",
  public = list(

    #' @field id (`character(1)`)\cr
    #'   Identifier of the object.
    id = NULL,

    #' @field values (`list()`)\cr
    #'   List of [paradox::TuneToken] that describe the tuning space and fixed parameter values.
    values = NULL,

    #' @field tags (`character()`)\cr
    #'   Arbitrary tags to group and filter tuning space e.g. `"classification"` or "`regression`".
    tags = NULL,

    #' @field learner (`character(1)`)\cr
    #'   [mlr3::Learner] of the tuning space.
    learner = NULL,

    #' @field package (`character(1)`)\cr
    #'   Packages which provide the [mlr3::Learner], e.g. \CRANpkg{mlr3learners} for the learner
    #'   [mlr3learners::LearnerClassifRanger] which interfaces the \CRANpkg{ranger} package.
    package = NULL,

    #' @field label (`character(1)`)\cr
    #'   Label for this object.
    #'   Can be used in tables, plot and text output instead of the ID.
    label = NULL,

    #' @field man (`character(1)`)\cr
    #'   String in the format `[pkg]::[topic]` pointing to a manual page for this object.
    #'   The referenced help package can be opened via method `$help()`.
    man = NULL,

    #' @description
    #' Creates a new instance of this [R6][R6::R6Class] class.
    #'
    #' @param id (`character(1)`)\cr
    #'   Identifier for the new instance.
    #'
    #' @param values (`list()`)\cr
    #'    List of [paradox::TuneToken] that describe the tuning space and fixed parameter values.
    #'
    #' @param tags (`character()`)\cr
    #'   Tags to group and filter tuning spaces e.g. `"classification"` or "`regression`".
    #'
    #' @param learner (`character(1)`)\cr
    #'   [mlr3::Learner] of the tuning space.
    #'
    #' @param package (`character()`)\cr
    #'   Packages which provide the [mlr3::Learner], e.g. \CRANpkg{mlr3learners} for the learner
    #'   [mlr3learners::LearnerClassifRanger] which interfaces the \CRANpkg{ranger} package.
    #'
    #' @param label (`character(1)`)\cr
    #'   Label for the new instance.
    #'   Can be used in tables, plot and text output instead of the ID.
    #'
    #' @param man (`character(1)`)\cr
    #'   String in the format `[pkg]::[topic]` pointing to a manual page for for the new instance.
    #'   The referenced help package can be opened via method `$help()`.
    initialize = function(id, values, tags, learner, package = character(), label = NA_character_, man = NA_character_) {
      self$id = assert_string(id, min.chars = 1L)
      self$learner = assert_string(learner, min.chars = 1L)
      self$values = assert_list(values, names = "unique")
      self$tags = assert_character(tags, any.missing = FALSE)
      self$package = union("mlr3tuningspaces", assert_character(package, any.missing = FALSE, min.chars = 1L))
      self$label = assert_string(label, na.ok = TRUE)
      self$man = assert_string(man, na.ok = TRUE)
    },

    #' @description
    #' Returns a learner with [paradox::TuneToken] set in parameter set.
    #'
    #' @param ... (named ‘list()’)\cr
    #'   Passed to `mlr3::lrn()`. Named arguments passed to the constructor, to
    #'   be set as parameters in the [paradox::ParamSet], or to be set as public
    #'   field. See `mlr3misc::dictionary_sugar_get()` for more details.
    #' @return [mlr3::Learner]
    get_learner = function(...) {
      learner = lrn(self$learner, ...)
      learner$param_set$values = insert_named(learner$param_set$values, self$values)
      learner
    },

    #' @description
    #' Helper for print outputs.
    #' @param ... (ignored).
    format = function(...) {
       sprintf("<%s:%s>", class(self)[1L], self$id)
    },

    #' @description
    #' Opens the corresponding help page referenced by field `$man`.
    help = function() {
      open_help(self$man)
    },

    #' @description
    #' Printer.
    #'
    #' @param ... (ignored).
    print = function(...) {
      tab = imap_dtr(self$values, function(value, name) {
        data.table(
            id = name,
            lower = value$content$lower,
            upper = value$content$upper,
            levels = list(value$content$param$levels),
            logscale = isTRUE(value$content$logscale)
          )
        }, .fill = TRUE)
      setcolorder(tab, c("id", "lower", "upper", "levels", "logscale"))

      msg_h =  if (is.null(self$label) || is.na(self$label)) "" else paste0(": ", self$label)
      cat_cli(cli_h1("{.cls {class(self)[1L]}} ({self$id}){msg_h}"))
      print(tab)
    }
  )
)

#' @include mlr_tuning_spaces.R
add_tuning_space = function(id, values, tags, learner, package = character(), label = NA_character_, man = NA_character_) { # nolint
  tuning_space = TuningSpace$new(id, values, tags, learner, package, label = label, man = paste0("mlr3tuningspaces::", id))
  mlr_tuning_spaces$add(id, tuning_space)
}

# rd_format_misc = function(lower, upper) {
#   if (is.na(lower) || is.na(upper) || is.null(lower) || is.null(upper)) return("-")

#   str = sprintf("%s%s, %s%s",
#     if (is.finite(lower)) "[" else "(",
#     if (is.finite(lower)) c(lower, lower) else c("-\\infty", "-Inf"),
#     if (is.finite(upper)) c(upper, upper) else c("\\infty", "Inf"),
#     if (is.finite(upper)) "]" else ")")
#   paste0("\\eqn{", str[1L], "}{", str[2L], "}")
# }

#' @export
rd_info.TuningSpace = function(obj, ...) { # nolint
  require_namespaces(obj$package)
  ps = lrn(obj$learner)$param_set
  x = c("",
    imap_chr(obj$values, function(space, name) {
      if (is.atomic(space)) {
        sprintf("* %s %s", name, space)
      } else if ("TuneToken" %nin% class(space)) {
        sprintf("* %s -", name)
      } else {
        switch(ps$params[name, , on = "id"]$cls,
          "ParamLgl" = sprintf("* %s \\[%s\\]", name, as_short_string(space$content$levels[[1]])),
          "ParamFct" = sprintf("* %s \\[%s\\]", name, rd_format_string(space$content$levels[[1]])),
          {lower = c(space$content$param$lower, space$content$lower) # one is NULL
          upper = c(space$content$upper, space$content$param$upper)
          logscale = if (is.null(space$content$logscale) || !space$content$logscale) character(1) else "Logscale"
          sprintf("* %s %s %s", name, rd_format_range(lower, upper), logscale)}
        )
      }
    })
  )
  paste(x, collapse = "\n")
}

#' @export
as_search_space.TuningSpace = function(x, ...) { # nolint
  x$get_learner()$param_set$search_space()
}

#' @export
as.data.table.TuningSpace = function(x, ...) {
  tab = map_dtr(x$values, function(value) {
    if (test_class(value, "ObjectTuneToken")) {
      # old paradox: value$content$param
      as.data.table(value$content$param %??% value$content)[, c("lower", "upper", "levels")]
    } else if (is.atomic(value)) {
      data.table(lower = NA, upper = NA, levels = NA, logscale = FALSE)
    } else if (is.function(value)) {
      data.table(lower = NA, upper = NA, levels = NA, logscale = FALSE)
    } else {
      as.data.table(value$content)
    }
  }, .fill = TRUE)
  tab[, "id" := names(x$values)]
  setcolorder(tab, intersect(c("id", "lower", "upper", "levels", "logscale"), names(tab)))
  if ("logscale" %in% names(tab)) tab[is.na(get("logscale")), "logscale" := FALSE]
  tab[]
}

# nolint start
library(mlr3)
library(mlr3misc)
library(mlr3learners)
library(mlr3torch)
library(paradox)
library(R6)

lapply(list.files(system.file("testthat", package = "mlr3"), pattern = "^helper.*\\.[rR]", full.names = TRUE), source)
# nolint end

test_tuning_space = function(key, learner) {
  learner = lrn(learner)
  learner$param_set$values = mlr_tuning_spaces$get(key)$values
  expect_equal_sorted(learner$param_set$values, mlr_tuning_spaces$get(key)$values)

  learner = mlr_tuning_spaces$get(key)$get_learner()
  expect_learner(learner)
  expect_names(names(learner$param_set$values), must.include = names(mlr_tuning_spaces$get(key)$values))
}

sortnames = function(x) {
  if (!is.null(names(x))) {
    x <- x[order(names(x), decreasing = TRUE)]
  }
  x
}

expect_equal_sorted = function(x, y, ...) {
  expect_equal(sortnames(x), sortnames(y), ...)
}

#' @title Dictionary of Tuning Spaces
#'
#' @usage NULL
#' @format [R6::R6Class] object inheriting from [mlr3misc::Dictionary].
#'
#' @description
#' A simple [mlr3misc::Dictionary] storing objects of class [TuningSpace].
#' Each tuning space has an associated help page, see `mlr_tuning_spaces_[id]`.
#'
#' @section Methods:
#' See [mlr3misc::Dictionary].
#'
#' @section S3 methods:
#' * `as.data.table(dict, ..., objects = FALSE)`\cr
#'   [mlr3misc::Dictionary] -> [data.table::data.table()]\cr
#'   Returns a [data.table::data.table()] with fields "key", "label", "learner", and "n_values" as columns.
#'   If `objects` is set to `TRUE`, the constructed objects are returned in the list column named `object`.
#'
#' @family Dictionary
#' @family TuningSpace
#' @export
#' @examples
#' as.data.table(mlr_tuning_spaces)
#' mlr_tuning_spaces$get("classif.ranger.default")
#' lts("classif.ranger.default")
mlr_tuning_spaces = R6Class("DictionaryTuningSpaces",
  inherit = Dictionary,
  cloneable = FALSE,
)$new()

#' @export
as.data.table.DictionaryTuningSpaces = function(x, ..., objects = FALSE) {
  assert_flag(objects)

  setkeyv(map_dtr(x$keys(), function(key) {
    t = withCallingHandlers(x$get(key),
      packageNotFoundWarning = function(w) invokeRestart("muffleWarning"))
    insert_named(
      list(key = key, label = t$label, learner = t$learner, n_values = length(t$values)),
      if (objects) list(object = list(t))
    )
  }, .fill = TRUE), "key")[]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TuningSpace.R
\name{TuningSpace}
\alias{TuningSpace}
\title{Tuning Spaces}
\description{
This class defines a tuning space for hyperparameter tuning.

For tuning, it is important to create a search space that defines the range over which hyperparameters should be tuned.
\code{TuningSpace} object consists of search spaces from peer-reviewed articles which work well for a wide range of data sets.

The \verb{$values} field stores a list of \link[paradox:to_tune]{paradox::TuneToken} which define the search space.
These tokens can be assigned to the \verb{$values} slot of a learner's \link[paradox:ParamSet]{paradox::ParamSet}.
When the learner is tuned, the tokens are used to create the search space.
}
\section{S3 Methods}{

\itemize{
\item \code{as.data.table.TuningSpace(x)}\cr
Returns a tabular view of the tuning space.\cr
\link{TuningSpace} -> \code{\link[data.table:data.table]{data.table::data.table()}}\cr
\itemize{
\item \code{x} (\link{TuningSpace})
}
}
}

\examples{
library(mlr3tuning)

# Get default tuning space of rpart learner
tuning_space = lts("classif.rpart.default")

# Set tuning space
learner = lrn("classif.rpart")
learner$param_set$values = tuning_space$values

# Tune learner
instance = tune(
  tnr("random_search"),
  task = tsk("sonar"),
  learner = learner,
  resampling = rsmp ("holdout"),
  measure = msr("classif.ce"),
  term_evals = 10)

instance$result

library(mlr3pipelines)

# Set tuning space in a pipeline
graph_learner = as_learner(po("subsample") \%>>\%
  lts(lrn("classif.rpart")))
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{id}}{(\code{character(1)})\cr
Identifier of the object.}

\item{\code{values}}{(\code{list()})\cr
List of \link[paradox:to_tune]{paradox::TuneToken} that describe the tuning space and fixed parameter values.}

\item{\code{tags}}{(\code{character()})\cr
Arbitrary tags to group and filter tuning space e.g. \code{"classification"} or "\code{regression}".}

\item{\code{learner}}{(\code{character(1)})\cr
\link[mlr3:Learner]{mlr3::Learner} of the tuning space.}

\item{\code{package}}{(\code{character(1)})\cr
Packages which provide the \link[mlr3:Learner]{mlr3::Learner}, e.g. \CRANpkg{mlr3learners} for the learner
\link[mlr3learners:mlr_learners_classif.ranger]{mlr3learners::LearnerClassifRanger} which interfaces the \CRANpkg{ranger} package.}

\item{\code{label}}{(\code{character(1)})\cr
Label for this object.
Can be used in tables, plot and text output instead of the ID.}

\item{\code{man}}{(\code{character(1)})\cr
String in the format \verb{[pkg]::[topic]} pointing to a manual page for this object.
The referenced help package can be opened via method \verb{$help()}.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-TuningSpace-new}{\code{TuningSpace$new()}}
\item \href{#method-TuningSpace-get_learner}{\code{TuningSpace$get_learner()}}
\item \href{#method-TuningSpace-format}{\code{TuningSpace$format()}}
\item \href{#method-TuningSpace-help}{\code{TuningSpace$help()}}
\item \href{#method-TuningSpace-print}{\code{TuningSpace$print()}}
\item \href{#method-TuningSpace-clone}{\code{TuningSpace$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TuningSpace-new"></a>}}
\if{latex}{\out{\hypertarget{method-TuningSpace-new}{}}}
\subsection{Method \code{new()}}{
Creates a new instance of this \link[R6:R6Class]{R6} class.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TuningSpace$new(
  id,
  values,
  tags,
  learner,
  package = character(),
  label = NA_character_,
  man = NA_character_
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{id}}{(\code{character(1)})\cr
Identifier for the new instance.}

\item{\code{values}}{(\code{list()})\cr
List of \link[paradox:to_tune]{paradox::TuneToken} that describe the tuning space and fixed parameter values.}

\item{\code{tags}}{(\code{character()})\cr
Tags to group and filter tuning spaces e.g. \code{"classification"} or "\code{regression}".}

\item{\code{learner}}{(\code{character(1)})\cr
\link[mlr3:Learner]{mlr3::Learner} of the tuning space.}

\item{\code{package}}{(\code{character()})\cr
Packages which provide the \link[mlr3:Learner]{mlr3::Learner}, e.g. \CRANpkg{mlr3learners} for the learner
\link[mlr3learners:mlr_learners_classif.ranger]{mlr3learners::LearnerClassifRanger} which interfaces the \CRANpkg{ranger} package.}

\item{\code{label}}{(\code{character(1)})\cr
Label for the new instance.
Can be used in tables, plot and text output instead of the ID.}

\item{\code{man}}{(\code{character(1)})\cr
String in the format \verb{[pkg]::[topic]} pointing to a manual page for for the new instance.
The referenced help package can be opened via method \verb{$help()}.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TuningSpace-get_learner"></a>}}
\if{latex}{\out{\hypertarget{method-TuningSpace-get_learner}{}}}
\subsection{Method \code{get_learner()}}{
Returns a learner with \link[paradox:to_tune]{paradox::TuneToken} set in parameter set.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TuningSpace$get_learner(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{(named ‘list()’)\cr
Passed to \code{mlr3::lrn()}. Named arguments passed to the constructor, to
be set as parameters in the \link[paradox:ParamSet]{paradox::ParamSet}, or to be set as public
field. See \code{mlr3misc::dictionary_sugar_get()} for more details.}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\link[mlr3:Learner]{mlr3::Learner}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TuningSpace-format"></a>}}
\if{latex}{\out{\hypertarget{method-TuningSpace-format}{}}}
\subsection{Method \code{format()}}{
Helper for print outputs.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TuningSpace$format(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{(ignored).}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TuningSpace-help"></a>}}
\if{latex}{\out{\hypertarget{method-TuningSpace-help}{}}}
\subsection{Method \code{help()}}{
Opens the corresponding help page referenced by field \verb{$man}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TuningSpace$help()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TuningSpace-print"></a>}}
\if{latex}{\out{\hypertarget{method-TuningSpace-print}{}}}
\subsection{Method \code{print()}}{
Printer.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TuningSpace$print(...)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{...}}{(ignored).}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-TuningSpace-clone"></a>}}
\if{latex}{\out{\hypertarget{method-TuningSpace-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{TuningSpace$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{package}
\name{mlr3tuningspaces-package}
\alias{mlr3tuningspaces}
\alias{mlr3tuningspaces-package}
\title{mlr3tuningspaces: Search Spaces for 'mlr3'}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Collection of search spaces for hyperparameter optimization in the 'mlr3' ecosystem. It features ready-to-use search spaces for many popular machine learning algorithms. The search spaces are from scientific articles and work for a wide range of data sets.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://mlr3tuningspaces.mlr-org.com}
  \item \url{https://github.com/mlr-org/mlr3tuningspaces}
  \item Report bugs at \url{https://github.com/mlr-org/mlr3tuningspaces/issues}
}

}
\author{
\strong{Maintainer}: Marc Becker \email{marcbecker@posteo.de} (\href{https://orcid.org/0000-0002-8115-0400}{ORCID})

Other contributors:
\itemize{
  \item Michel Lang \email{michellang@gmail.com} (\href{https://orcid.org/0000-0001-9754-0393}{ORCID}) [contributor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tuning_spaces_rbv1.R
\name{mlr_tuning_spaces_rbv1}
\alias{mlr_tuning_spaces_rbv1}
\alias{mlr_tuning_spaces_classif.glmnet.rbv1}
\alias{mlr_tuning_spaces_classif.ranger.rbv1}
\alias{mlr_tuning_spaces_classif.rpart.rbv1}
\alias{mlr_tuning_spaces_classif.svm.rbv1}
\alias{mlr_tuning_spaces_classif.xgboost.rbv1}
\alias{mlr_tuning_spaces_regr.glmnet.rbv1}
\alias{mlr_tuning_spaces_regr.ranger.rbv1}
\alias{mlr_tuning_spaces_regr.rpart.rbv1}
\alias{mlr_tuning_spaces_regr.svm.rbv1}
\alias{mlr_tuning_spaces_regr.xgboost.rbv1}
\title{RandomBot Tuning Spaces}
\source{
Kuehn D, Probst P, Thomas J, Bischl B (2018).
\dQuote{Automatic Exploration of Machine Learning Experiments on OpenML.}
1806.10961, \url{https://arxiv.org/abs/1806.10961}.
}
\description{
Tuning spaces from the Kuehn (2018) article.
}
\section{glmnet tuning space}{

\itemize{
\item alpha \eqn{[0, 1]}{[0, 1]}
\item s \eqn{[1e-04, 1000]}{[1e-04, 1000]} Logscale
}
}

\section{ranger tuning space}{

\itemize{
\item num.trees \eqn{[1, 2000]}{[1, 2000]}
\item replace [TRUE,FALSE]
\item sample.fraction \eqn{[0.1, 1]}{[0.1, 1]}
\item mtry.ratio \eqn{[0, 1]}{[0, 1]}
\item respect.unordered.factors [\dQuote{ignore}, \dQuote{order}]
\item min.node.size \eqn{[1, 100]}{[1, 100]}
}

The tuning space of the ranger learner is slightly different from the original paper.
The hyperparameter \code{mtry.power} is replaced by \code{mtry.ratio} and \code{min.node.size} is explored in a range from 1 to 100.
}

\section{rpart tuning space}{

\itemize{
\item cp \eqn{[0, 1]}{[0, 1]}
\item maxdepth \eqn{[1, 30]}{[1, 30]}
\item minbucket \eqn{[1, 60]}{[1, 60]}
\item minsplit \eqn{[1, 60]}{[1, 60]}
}
}

\section{svm tuning space}{

\itemize{
\item kernel [\dQuote{linear}, \dQuote{polynomial}, \dQuote{radial}]
\item cost \eqn{[1e-04, 1000]}{[1e-04, 1000]} Logscale
\item gamma \eqn{[1e-04, 1000]}{[1e-04, 1000]} Logscale
\item degree \eqn{[2, 5]}{[2, 5]}
}
}

\section{xgboost tuning space}{

\itemize{
\item nrounds \eqn{[1, 5000]}{[1, 5000]}
\item eta \eqn{[1e-04, 1]}{[1e-04, 1]} Logscale
\item subsample \eqn{[0, 1]}{[0, 1]}
\item booster [\dQuote{gblinear}, \dQuote{gbtree}, \dQuote{dart}]
\item max_depth \eqn{[1, 15]}{[1, 15]}
\item min_child_weight \eqn{[1, 100]}{[1, 100]} Logscale
\item colsample_bytree \eqn{[0, 1]}{[0, 1]}
\item colsample_bylevel \eqn{[0, 1]}{[0, 1]}
\item lambda \eqn{[1e-04, 1000]}{[1e-04, 1000]} Logscale
\item alpha \eqn{[1e-04, 1000]}{[1e-04, 1000]} Logscale
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tuning_spaces_default.R
\name{mlr_tuning_spaces_default}
\alias{mlr_tuning_spaces_default}
\alias{mlr_tuning_spaces_classif.glmnet.default}
\alias{mlr_tuning_spaces_classif.ranger.default}
\alias{mlr_tuning_spaces_classif.rpart.default}
\alias{mlr_tuning_spaces_classif.svm.default}
\alias{mlr_tuning_spaces_classif.xgboost.default}
\alias{mlr_tuning_spaces_regr.glmnet.default}
\alias{mlr_tuning_spaces_regr.ranger.default}
\alias{mlr_tuning_spaces_regr.rpart.default}
\alias{mlr_tuning_spaces_regr.svm.default}
\alias{mlr_tuning_spaces_regr.xgboost.default}
\title{Default Tuning Spaces}
\source{
Bischl B, Binder M, Lang M, Pielok T, Richter J, Coors S, Thomas J, Ullmann T, Becker M, Boulesteix A, Deng D, Lindauer M (2023).
\dQuote{Hyperparameter Optimization: Foundations, Algorithms, Best Practices and Open Challenges.}
}
\description{
Tuning spaces from the Bischl (2023) article.
}
\section{glmnet tuning space}{

\itemize{
\item s \eqn{[1e-04, 10000]}{[1e-04, 10000]} Logscale
\item alpha \eqn{[0, 1]}{[0, 1]}
}
}

\section{ranger tuning space}{

\itemize{
\item mtry.ratio \eqn{[0, 1]}{[0, 1]}
\item replace [TRUE,FALSE]
\item sample.fraction \eqn{[0.1, 1]}{[0.1, 1]}
\item num.trees \eqn{[1, 2000]}{[1, 2000]}
}
}

\section{rpart tuning space}{

\itemize{
\item minsplit \eqn{[2, 128]}{[2, 128]} Logscale
\item minbucket \eqn{[1, 64]}{[1, 64]} Logscale
\item cp \eqn{[1e-04, 0.1]}{[1e-04, 0.1]} Logscale
}
}

\section{svm tuning space}{

\itemize{
\item cost \eqn{[1e-04, 10000]}{[1e-04, 10000]} Logscale
\item kernel [\dQuote{polynomial}, \dQuote{radial}, \dQuote{sigmoid}, \dQuote{linear}]
\item degree \eqn{[2, 5]}{[2, 5]}
\item gamma \eqn{[1e-04, 10000]}{[1e-04, 10000]} Logscale
}
}

\section{xgboost tuning space}{

\itemize{
\item eta \eqn{[1e-04, 1]}{[1e-04, 1]} Logscale
\item nrounds \eqn{[1, 5000]}{[1, 5000]}
\item max_depth \eqn{[1, 20]}{[1, 20]}
\item colsample_bytree \eqn{[0.1, 1]}{[0.1, 1]}
\item colsample_bylevel \eqn{[0.1, 1]}{[0.1, 1]}
\item lambda \eqn{[0.001, 1000]}{[0.001, 1000]} Logscale
\item alpha \eqn{[0.001, 1000]}{[0.001, 1000]} Logscale
\item subsample \eqn{[0.1, 1]}{[0.1, 1]}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tuning_spaces_rtdl.R
\name{mlr_tuning_spaces_rtdl}
\alias{mlr_tuning_spaces_rtdl}
\alias{mlr_tuning_spaces_classif.mlp.rtdl}
\alias{mlr_tuning_spaces_classif.tab_resnet.rtdl}
\alias{mlr_tuning_spaces_classif.ft_transformer.rtdl}
\alias{mlr_tuning_spaces_regr.mlp.rtdl}
\alias{mlr_tuning_spaces_regr.tab_resnet.rtdl}
\alias{mlr_tuning_spaces_regr.ft_transformer.rtdl}
\title{Deep Learning Tuning Spaces from Yandex's RTDL}
\source{
Gorishniy Y, Rubachev I, Khrulkov V, Babenko A (2021).
\dQuote{Revisiting Deep Learning  for Tabular Data.}
\emph{arXiv}, \bold{2106.11959}.
}
\description{
Tuning spaces for deep neural network architectures from the Gorishniy (2021) article.

These tuning spaces require optimizers that have a \code{weight_decay} parameter, such as AdamW or any of the other optimizers built into \code{mlr3torch}.

When the article suggests multiple ranges for a given hyperparameter, these tuning spaces choose the widest range.

The FT-Transformer tuning space disables weight decay for all bias parameters, matching the implementation provided by the authors in the rtdl-revisiting-models package.
However, this differs from the experiments described in the article, which states that the

For the FT-Transformer, if training is unstable, consider a combination of standardizing features, using an adaptive optimizer (e.g. Adam), reducing the learning rate,
and using a learning rate scheduler.
}
\section{MLP tuning space}{

\itemize{
\item n_layers \eqn{[1, 16]}{[1, 16]}
\item neurons -
\item p \eqn{[0, 0.5]}{[0, 0.5]}
\item opt.lr \eqn{[1e-05, 0.01]}{[1e-05, 0.01]} Logscale
\item opt.weight_decay \eqn{[1e-06, 0.001]}{[1e-06, 0.001]} Logscale
\item epochs \eqn{[1, 100]}{[1, 100]}
\item patience 17
}
}

\section{Tabular ResNet tuning space}{

\itemize{
\item n_blocks \eqn{[1, 16]}{[1, 16]}
\item d_block \eqn{[64, 1024]}{[64, 1024]}
\item d_hidden_multiplier \eqn{[1, 4]}{[1, 4]}
\item dropout1 \eqn{[0, 0.5]}{[0, 0.5]}
\item dropout2 \eqn{[0, 0.5]}{[0, 0.5]}
\item opt.lr \eqn{[1e-05, 0.01]}{[1e-05, 0.01]} Logscale
\item opt.weight_decay \eqn{[1e-06, 0.001]}{[1e-06, 0.001]} Logscale
\item epochs \eqn{[1, 100]}{[1, 100]}
\item patience 17
}
}

\section{FT-Transformer tuning space}{

\itemize{
\item n_blocks \eqn{[1, 6]}{[1, 6]}
\item d_token \eqn{[8, 64]}{[8, 64]}
\item attention_n_heads 8
\item residual_dropout \eqn{[0, 0.2]}{[0, 0.2]}
\item attention_dropout \eqn{[0, 0.5]}{[0, 0.5]}
\item ffn_dropout \eqn{[0, 0.5]}{[0, 0.5]}
\item ffn_d_hidden_multiplier \eqn{[0.666666666666667, 2.66666666666667]}{[0.666666666666667, 2.66666666666667]}
\item opt.lr \eqn{[1e-05, 1e-04]}{[1e-05, 1e-04]} Logscale
\item opt.weight_decay \eqn{[1e-06, 0.001]}{[1e-06, 0.001]} Logscale
\item opt.param_groups -
\item epochs \eqn{[1, 100]}{[1, 100]}
\item patience 17
}

In the FT-Transformer, the validation-related parameters must still be set manually, via e.g. \code{lts("regr.ft_transformer.rtdl")$get_learner(validate = 0.2, measures_valid = msr("regr.rmse"))}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mlr_tuning_spaces.R
\docType{data}
\name{mlr_tuning_spaces}
\alias{mlr_tuning_spaces}
\title{Dictionary of Tuning Spaces}
\format{
\link[R6:R6Class]{R6::R6Class} object inheriting from \link[mlr3misc:Dictionary]{mlr3misc::Dictionary}.
}
\description{
A simple \link[mlr3misc:Dictionary]{mlr3misc::Dictionary} storing objects of class \link{TuningSpace}.
Each tuning space has an associated help page, see \code{mlr_tuning_spaces_[id]}.
}
\section{Methods}{

See \link[mlr3misc:Dictionary]{mlr3misc::Dictionary}.
}

\section{S3 methods}{

\itemize{
\item \code{as.data.table(dict, ..., objects = FALSE)}\cr
\link[mlr3misc:Dictionary]{mlr3misc::Dictionary} -> \code{\link[data.table:data.table]{data.table::data.table()}}\cr
Returns a \code{\link[data.table:data.table]{data.table::data.table()}} with fields "key", "label", "learner", and "n_values" as columns.
If \code{objects} is set to \code{TRUE}, the constructed objects are returned in the list column named \code{object}.
}
}

\examples{
as.data.table(mlr_tuning_spaces)
mlr_tuning_spaces$get("classif.ranger.default")
lts("classif.ranger.default")
}
\concept{Dictionary}
\concept{TuningSpace}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sugar.R
\name{lts}
\alias{lts}
\alias{lts.missing}
\alias{lts.character}
\alias{lts.Learner}
\alias{ltss}
\title{Syntactic Sugar for Tuning Space Construction}
\usage{
lts(x, ...)

\method{lts}{missing}(x, ...)

\method{lts}{character}(x, ...)

\method{lts}{Learner}(x, ...)

ltss(x)
}
\arguments{
\item{x}{(\code{character()} | \link[mlr3:Learner]{mlr3::Learner})\cr
If \code{character}, key passed the dictionary to retrieve the tuning space.
If \link[mlr3:Learner]{mlr3::Learner}, default tuning space is added to the learner.}

\item{...}{(named list of \link[paradox:to_tune]{paradox::TuneToken} | \code{NULL})\cr
Pass \link[paradox:to_tune]{paradox::TuneToken} to add or overwrite parameters in the tuning space.
Use \code{NULL} to remove parameters (see examples).}
}
\value{
\link{TuningSpace} if \code{x} is \code{character()}.
\link[mlr3:Learner]{mlr3::Learner} if \code{x} is \link[mlr3:Learner]{mlr3::Learner}.
Or a list of objects for the \code{ltss()} function.

missing, \link{mlr_tuning_spaces} dictionary

a \code{character}, \link{TuningSpace}

a \link[mlr3:Learner]{mlr3::Learner}, \link[mlr3:Learner]{mlr3::Learner} with \link[paradox:to_tune]{paradox::TuneToken}

a \code{list()}, list of \link{TuningSpace} or \link[mlr3:Learner]{mlr3::Learner}
}
\description{
Function to retrieve \link{TuningSpace} objects from \link{mlr_tuning_spaces} and further, allows a \link[mlr3:Learner]{mlr3::Learner} to be directly configured with a search space.
This function belongs to \link[mlr3:mlr_sugar]{mlr3::mlr_sugar} family.
}
\examples{
# load tuning space
lts("classif.rpart.default")

# load tuning space and add parameter
lts("classif.rpart.default", maxdepth = to_tune(1, 15))

# load tuning space and remove parameter
lts("classif.rpart.default", minsplit = NULL)

# load tuning space and overwrite parameter
lts("classif.rpart.default", minsplit = to_tune(32, 128))

# load learner and apply tuning space in one go
lts(lrn("classif.rpart"))

# load learner, overwrite parameter and apply tuning space
lts(lrn("classif.rpart"), minsplit = to_tune(32, 128))

# load multiple tuning spaces
ltss(c("classif.rpart.default", "classif.ranger.default"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tuning_spaces_rbv2.R
\name{mlr_tuning_spaces_rbv2}
\alias{mlr_tuning_spaces_rbv2}
\alias{mlr_tuning_spaces_classif.glmnet.rbv2}
\alias{mlr_tuning_spaces_classif.ranger.rbv2}
\alias{mlr_tuning_spaces_classif.rpart.rbv2}
\alias{mlr_tuning_spaces_classif.svm.rbv2}
\alias{mlr_tuning_spaces_classif.xgboost.rbv2}
\alias{mlr_tuning_spaces_regr.glmnet.rbv2}
\alias{mlr_tuning_spaces_regr.ranger.rbv2}
\alias{mlr_tuning_spaces_regr.rpart.rbv2}
\alias{mlr_tuning_spaces_regr.svm.rbv2}
\alias{mlr_tuning_spaces_regr.xgboost.rbv2}
\title{RandomBot V2 Tuning Spaces}
\source{
Binder M, Pfisterer F, Bischl B (2020).
\dQuote{Collecting Empirical Data About Hyperparameters for Data Driven AutoML.}
\url{https://www.automl.org/wp-content/uploads/2020/07/AutoML_2020_paper_63.pdf}.
}
\description{
Tuning spaces from the Binder (2020) article.
}
\section{glmnet tuning space}{

\itemize{
\item alpha \eqn{[0, 1]}{[0, 1]}
\item s \eqn{[1e-04, 1000]}{[1e-04, 1000]} Logscale
}
}

\section{ranger tuning space}{

\itemize{
\item num.trees \eqn{[1, 2000]}{[1, 2000]}
\item replace [TRUE,FALSE]
\item sample.fraction \eqn{[0.1, 1]}{[0.1, 1]}
\item mtry.ratio \eqn{[0, 1]}{[0, 1]}
\item respect.unordered.factors [\dQuote{ignore}, \dQuote{order}, \dQuote{partition}]
\item min.node.size \eqn{[1, 100]}{[1, 100]}
\item splitrule [\dQuote{gini}, \dQuote{extratrees}]
\item num.random.splits \eqn{[1, 100]}{[1, 100]}
}

\code{mtry.power} is replaced by \code{mtry.ratio}.
}

\section{rpart tuning space}{

\itemize{
\item cp \eqn{[1e-04, 1]}{[1e-04, 1]} Logscale
\item maxdepth \eqn{[1, 30]}{[1, 30]}
\item minbucket \eqn{[1, 100]}{[1, 100]}
\item minsplit \eqn{[1, 100]}{[1, 100]}
}
}

\section{svm tuning space}{

\itemize{
\item kernel [\dQuote{linear}, \dQuote{polynomial}, \dQuote{radial}]
\item cost \eqn{[1e-04, 1000]}{[1e-04, 1000]} Logscale
\item gamma \eqn{[1e-04, 1000]}{[1e-04, 1000]} Logscale
\item tolerance \eqn{[1e-04, 2]}{[1e-04, 2]} Logscale
\item degree \eqn{[2, 5]}{[2, 5]}
}
}

\section{xgboost tuning space}{

\itemize{
\item booster [\dQuote{gblinear}, \dQuote{gbtree}, \dQuote{dart}]
\item nrounds \eqn{[7, 2981]}{[7, 2981]} Logscale
\item eta \eqn{[1e-04, 1]}{[1e-04, 1]} Logscale
\item gamma \eqn{[1e-05, 7]}{[1e-05, 7]} Logscale
\item lambda \eqn{[1e-04, 1000]}{[1e-04, 1000]} Logscale
\item alpha \eqn{[1e-04, 1000]}{[1e-04, 1000]} Logscale
\item subsample \eqn{[0.1, 1]}{[0.1, 1]}
\item max_depth \eqn{[1, 15]}{[1, 15]}
\item min_child_weight \eqn{[1, 100]}{[1, 100]} Logscale
\item colsample_bytree \eqn{[0.01, 1]}{[0.01, 1]}
\item colsample_bylevel \eqn{[0.01, 1]}{[0.01, 1]}
\item rate_drop \eqn{[0, 1]}{[0, 1]}
\item skip_drop \eqn{[0, 1]}{[0, 1]}
}
}
